<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', '../');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';


if ($pun_user['g_id'] != PUN_ADMIN)
	message($lang_common['No permission']);

// Load the admin.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/admin.php';


// Add a new category
if (isset($_POST['add_cat']))
{
	$new_cat_name = trim($_POST['new_cat_name']);
	if ($new_cat_name == '')
		message($lang_admin['Must name category']);

	$db->query('INSERT INTO '.$db->prefix.'categories (cat_name) VALUES(\''.$db->escape($new_cat_name).'\')') or error('Unable to create category', __FILE__, __LINE__, $db->error());

	redirect(pun_link($pun_url['admin_categories']), $lang_admin['Category added'].' '.$lang_admin['Redirect']);
}


// Delete a category
else if (isset($_POST['del_cat']) || isset($_POST['del_cat_comply']))
{
	$cat_to_delete = intval($_POST['cat_to_delete']);
	if ($cat_to_delete < 1)
		message($lang_common['Bad request']);

	// User pressed the cancel button
	if (isset($_POST['del_cat_cancel']))
		redirect(pun_link($pun_url['admin_categories']), $lang_admin['Cancel redirect']);

	if (isset($_POST['del_cat_comply']))	// Delete a category with all forums and posts
	{
		@set_time_limit(0);

		$result = $db->query('SELECT id FROM '.$db->prefix.'forums WHERE cat_id='.$cat_to_delete) or error('Unable to fetch forum list', __FILE__, __LINE__, $db->error());
		$num_forums = $db->num_rows($result);

		for ($i = 0; $i < $num_forums; ++$i)
		{
			$cur_forum = $db->result($result, $i);

			// Prune all posts and topics
			prune($cur_forum, 1, -1);

			// Delete the forum
			$db->query('DELETE FROM '.$db->prefix.'forums WHERE id='.$cur_forum) or error('Unable to delete forum', __FILE__, __LINE__, $db->error());
		}

		// Locate any "orphaned redirect topics" and delete them
		$result = $db->query('SELECT t1.id FROM '.$db->prefix.'topics AS t1 LEFT JOIN '.$db->prefix.'topics AS t2 ON t1.moved_to=t2.id WHERE t2.id IS NULL AND t1.moved_to IS NOT NULL') or error('Unable to fetch redirect topics', __FILE__, __LINE__, $db->error());
		$num_orphans = $db->num_rows($result);

		if ($num_orphans)
		{
			for ($i = 0; $i < $num_orphans; ++$i)
				$orphans[] = $db->result($result, $i);

			$db->query('DELETE FROM '.$db->prefix.'topics WHERE id IN('.implode(',', $orphans).')') or error('Unable to delete redirect topics', __FILE__, __LINE__, $db->error());
		}

		// Delete the category
		$db->query('DELETE FROM '.$db->prefix.'categories WHERE id='.$cat_to_delete) or error('Unable to delete category', __FILE__, __LINE__, $db->error());

		// Regenerate the quickjump cache
		require_once PUN_ROOT.'include/cache.php';
		generate_quickjump_cache();

		redirect(pun_link($pun_url['admin_categories']), $lang_admin['Category deleted'].' '.$lang_admin['Redirect']);
	}
	else	// If the user hasn't comfirmed the delete
	{
		$result = $db->query('SELECT cat_name FROM '.$db->prefix.'categories WHERE id='.$cat_to_delete) or error('Unable to fetch category info', __FILE__, __LINE__, $db->error());
		$cat_name = $db->result($result);


		// Setup breadcrumbs
		$pun_page['crumbs'] = array(
			array($pun_config['o_board_title'], pun_link($pun_url['index'])),
			array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
			array($lang_admin['Categories'], pun_link($pun_url['admin_categories'])),
			$lang_admin['Delete category']
		);

		define('PUN_PAGE_SECTION', 'setup');
		define('PUN_PAGE', 'admin-categories');
		require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_categories']) ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="cat_to_delete" value="<?php echo $cat_to_delete ?>" />
				<div class="form-head">
					<h2><span><?php echo end($pun_page['crumbs']) ?> "<?php echo htmlspecialchars($cat_name) ?>"</span></h2>
				</div>
				<div class="form-warn">
					<p><?php echo $lang_admin['Delete category confirm'] ?> "<?php echo htmlspecialchars($cat_name) ?>"?</p>
					<p><strong class="warntext"><?php echo $lang_admin['Delete category warning'] ?></strong></p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="del_cat_comply" value="<?php echo $lang_admin['Delete'] ?>" /> <input type="submit" class="button" name="del_cat_cancel" value="<?php echo $lang_admin['Cancel'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

		require PUN_ROOT.'footer.php';
	}
}


else if (isset($_POST['update']))	// Change position and name of the categories
{
	$cat_order = array_map("intval", $_POST['cat_order']);
	$cat_name = array_map("trim", $_POST['cat_name']);

	$result = $db->query('SELECT id, cat_name, disp_position FROM '.$db->prefix.'categories ORDER BY id ASC') or error('Unable to fetch category list', __FILE__, __LINE__, $db->error());

	while ($cur_cat = $db->fetch_assoc($result))
	{
		// If these aren't set, we're looking at a category that was added after
		// the admin started editing: we don't want to mess with it
		if (isset($cat_name[$cur_cat['id']]) && isset($cat_order[$cur_cat['id']]))
		{
			if ($cat_name[$cur_cat['id']] == '')
				message($lang_admin['Must enter category']);

			if ($cat_order[$cur_cat['id']] < 0)
				message($lang_admin['Must be integer']);

			// We only want to update if we changed anything
			if ($cur_cat['cat_name'] != $cat_name[$cur_cat['id']] || $cur_cat['disp_position'] != $cat_order[$cur_cat['id']])
				$db->query('UPDATE '.$db->prefix.'categories SET cat_name=\''.$db->escape($cat_name[$cur_cat['id']]).'\', disp_position='.$cat_order[$cur_cat['id']].' WHERE id='.$cur_cat['id']) or error('Unable to update category', __FILE__, __LINE__, $db->error());
		}
	}

	// Regenerate the quickjump cache
	require_once PUN_ROOT.'include/cache.php';
	generate_quickjump_cache();

	redirect(pun_link($pun_url['admin_categories']), $lang_admin['Categories updated'].' '.$lang_admin['Redirect']);
}


// Generate an array with all categories
$result = $db->query('SELECT id, cat_name, disp_position FROM '.$db->prefix.'categories ORDER BY disp_position') or error('Unable to fetch category list', __FILE__, __LINE__, $db->error());
$num_cats = $db->num_rows($result);

for ($i = 0; $i < $num_cats; ++$i)
	$cat_list[] = $db->fetch_row($result);


// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
	$lang_admin['Categories']
);

define('PUN_PAGE_SECTION', 'setup');
define('PUN_PAGE', 'admin-categories');
require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_categories']) ?>?action=foo">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Add category head'] ?>: </span></h2>
					<p><?php echo $lang_admin['New category intro'] ?> <a href="<?php echo pun_link($pun_url['admin_forums']) ?>"><?php echo $lang_admin['Forums'] ?>.</a></p>
				</div>
				<div class="fields textset" id="fs1">
					<p class="field">
						<label for="fs1f1"><?php echo $lang_admin['New category name'] ?></label><br />
						<span class="input"><input type="text" id="fs1f1" name="new_cat_name" size="35" maxlength="80" /></span>
					</p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="add_cat" value="<?php echo $lang_admin['Add new'] ?>" /></p>
			</div>

<?php if ($num_cats): ?>			<div class="divider"><hr /></div>
			<div class="panel">
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Delete category'] ?>: </span></h2>
					<p><?php echo $lang_admin['Delete category intro'] ?></p>
				</div>
				<div class="fields textset" id="fs2">
					<p class="field">
						<label for="fs2f1"><?php echo $lang_admin['Category to delete'] ?></label><br />
						<span class="input"><select id="fs2f1" name="cat_to_delete">
<?php

	while (list(, list($cat_id, $cat_name, ,)) = @each($cat_list))
		echo "\t\t\t\t\t\t\t".'<option value="'.$cat_id.'">'.htmlspecialchars($cat_name).'</option>'."\n";

?>
						</select></span>
					</p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="del_cat" value="<?php echo $lang_admin['Delete'] ?>" /></p>
			</div>
<?php endif; ?>			</form>
<?php

if ($num_cats)
{

?>
			<div class="divider"><hr /></div>
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_categories']) ?>?action=foo">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Edit categories head'] ?>: </span></h2>
					<p><?php echo $lang_admin['Edit categories intro'] ?></p>
				</div>
<?php

	@reset($cat_list);
	for ($i = 0; $i < $num_cats; ++$i)
	{
		list(, list($cat_id, $cat_name, $position)) = @each($cat_list);

?>
				<fieldset class="textset" id="fs<?php echo ($i + 3) ?>">
					<legend><span><?php echo $lang_admin['Category'].' '.$position ?></span></legend>
					<p class="field">
						<label for="fs<?php echo ($i + 3) ?>f1"><?php echo $lang_admin['Category'] ?></label><br />
						<span class="input"><input type="text" id="fs<?php echo ($i + 3) ?>f1" name="cat_name[<?php echo $cat_id ?>]" value="<?php echo htmlspecialchars($cat_name) ?>" size="35" maxlength="80" /></span>
					</p>
					<p class="field">
						<label for="fs<?php echo ($i + 3) ?>f2"><?php echo $lang_admin['Position'] ?></label><br />
						<span class="input"><input type="text" id="fs<?php echo ($i + 3) ?>f2" name="cat_order[<?php echo $cat_id ?>]" value="<?php echo $position ?>" size="3" maxlength="3" /></span>
					</p>
				</fieldset>
<?php

	}

?>
				<p class="formsubmit"><input type="submit" class="button" name="update" value="<?php echo $lang_admin['Update all'] ?>" /></p>
			</div>
			</form>
		</div>
<?php

}

?>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
