<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', '../');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';
require PUN_ROOT.'include/xml.php';


if ($pun_user['g_id'] != PUN_ADMIN)
	message($lang_common['No permission']);

// Load the admin.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/admin.php';


// Install an extension
if (isset($_GET['install']))
{
	$id = preg_replace('/[^a-zA-Z_]/', '', $_GET['install']);

	// Parse manifest.xml into an array and validate it
	$ext_data = xml_to_array(@file_get_contents(PUN_ROOT.'extensions/'.$id.'/manifest.xml'));
	$errors = validate_manifest($ext_data, $id);

	if (!empty($errors))
		message($lang_common['Bad request']);

	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
		array($lang_admin['Manage extensions'], pun_link($pun_url['admin_extensions'])),
		$lang_admin['Install extension']
	);

	if (isset($_POST['install_comply']))
	{
		$notes = array();
		
		$new_uninstall_code = (isset($ext_data['extension']['uninstall']) && trim($ext_data['extension']['uninstall']) != '') ? '\''.$db->escape(trim($ext_data['extension']['uninstall'])).'\'' : 'NULL';

		// Is this a fresh install or an upgrade?
		$result = $db->query('SELECT version FROM '.$db->prefix.'extensions WHERE id=\''.$db->escape($id).'\'') or error('Unable to fetch extension', __FILE__, __LINE__, $db->error());
		if ($db->num_rows($result))
		{
			// $cur_version will be available to the extension install routine (to facilitate extension upgrades)
			$cur_version = $db->result($result);

			// Run the author supplied install code
			if (isset($ext_data['extension']['install']) && trim($ext_data['extension']['install']) != '')
				eval($ext_data['extension']['install']);

			$db->query('UPDATE '.$db->prefix.'extensions SET title=\''.$db->escape($ext_data['extension']['title']).'\', version=\''.$db->escape($ext_data['extension']['version']).'\', description=\''.$db->escape($ext_data['extension']['description']).'\', author=\''.$db->escape($ext_data['extension']['author']).'\', uninstall='.$new_uninstall_code.' WHERE id=\''.$db->escape($id).'\'') or error('Unable to update extension', __FILE__, __LINE__, $db->error());
			$db->query('DELETE FROM '.$db->prefix.'extension_hooks WHERE extension_id=\''.$db->escape($id).'\'') or error('Unable to delete extension hooks', __FILE__, __LINE__, $db->error());
		}
		else
		{
			// Run the author supplied install code
			if (isset($ext_data['extension']['install']) && trim($ext_data['extension']['install']) != '')
				eval($ext_data['extension']['install']);

			$db->query('INSERT INTO '.$db->prefix.'extensions (id, title, version, description, author, uninstall) VALUES(\''.$db->escape($ext_data['extension']['id']).'\', \''.$db->escape($ext_data['extension']['title']).'\', \''.$db->escape($ext_data['extension']['version']).'\', \''.$db->escape($ext_data['extension']['description']).'\', \''.$db->escape($ext_data['extension']['author']).'\', '.$new_uninstall_code.')') or error('Unable to create extension', __FILE__, __LINE__, $db->error());
		}

		// Now insert the hooks
		$hooks = $ext_data['extension']['hooks']['hook'];
		if (!is_array(current($hooks)))
			$hooks = array($hooks);

		foreach ($hooks as $hook)
			$db->query('INSERT INTO '.$db->prefix.'extension_hooks (id, extension_id, code, installed) VALUES(\''.$db->escape(trim($hook['attributes']['id'])).'\', \''.$db->escape($id).'\', \''.$db->escape(trim($hook['content'])).'\', '.time().')') or error('Unable to create extension hook', __FILE__, __LINE__, $db->error());

		// Empty the PHP cache
		$d = dir(PUN_ROOT.'cache');
		while (($entry = $d->read()) !== false)
		{
			if (substr($entry, strlen($entry)-4) == '.php')
				@unlink(PUN_ROOT.'cache/'.$entry);
		}
		$d->close();

		// Regenerate the hooks cache
		require_once PUN_ROOT.'include/cache.php';
		generate_hooks_cache();

		// Display notes if there's any
		if (!empty($notes))
		{
			define('PUN_PAGE_SECTION', 'extensions');
			define('PUN_PAGE', 'admin-extensions');
			require PUN_ROOT.'header.php';
?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="form-warn">
				<p><strong><?php echo $lang_admin['Install notes'] ?></strong></p>
				<ul class="warnlist">
<?php

	while (list(, $cur_note) = each($notes))
		echo "\t\t\t\t\t".'<li><span>'.$cur_note.'</span></li>'."\n";

?>
				</ul>
			</div>
		</div>
	</div>

</div>
<?php
			require PUN_ROOT.'footer.php';

		}
		else
			redirect(pun_link($pun_url['admin_extensions']), $lang_admin['Extension installed'].' '.$lang_admin['Redirect']);
	}

	define('PUN_PAGE_SECTION', 'extensions');
	define('PUN_PAGE', 'admin-extensions');
	require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>?install=<?php echo $id ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2><span><?php echo end($pun_page['crumbs']) ?></span></h2>
				</div>
				<div class="textset">
					<h3 class="field"><strong class="alabel"><?php echo $lang_admin['Extension title'] ?></strong> <span class="input"><?php echo htmlspecialchars($ext_data['extension']['title']) ?></span></h3>
					<ul class="field">
						<li><strong class="alabel"><?php echo $lang_admin['Extension version'] ?></strong> <span class="input"><?php echo $ext_data['extension']['version'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension author'] ?></strong> <span class="input"><?php echo $ext_data['extension']['author'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension description'] ?></strong> <span class="input"><?php echo htmlspecialchars($ext_data['extension']['description']) ?></span></li>
					</ul>
				</div>
				<p><strong><?php echo $lang_admin['Extension overwrite warning'] ?></strong></p>
				<p class="formsubmit"><input type="submit" name="install_comply" value="<?php echo $lang_admin['Install'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

	require PUN_ROOT.'footer.php';
}


// Uninstall an extension
else if (isset($_POST['uninstall']))
{
	$id = key($_POST['uninstall']);

	// Fetch info about the extension
	$result = $db->query('SELECT title, version, description, author, uninstall FROM '.$db->prefix.'extensions WHERE id=\''.$db->escape($id).'\'') or error('Unable to fetch extension', __FILE__, __LINE__, $db->error());
	if (!$db->num_rows($result))
		message($lang_common['Bad request']);

	$ext_data = $db->fetch_assoc($result);

	if (!isset($_POST['verify']))
	{

		// Setup breadcrumbs
		$pun_page['crumbs'] = array(
			array($pun_config['o_board_title'], pun_link($pun_url['index'])),
			array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
			array($lang_admin['Manage extensions'], pun_link($pun_url['admin_extensions'])),
			$lang_admin['Uninstall extension']
		);

		define('PUN_PAGE_SECTION', 'extensions');
		define('PUN_PAGE', 'admin-extensions');
		require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="verify" value="1" />
				<div class="form-intro">
					<h2><span><?php echo end($pun_page['crumbs']) ?></span></h2>
				</div>
				<div class="textset">
					<h3 class="field"><strong class="alabel"><?php echo $lang_admin['Extension title'] ?></strong> <span class="input"><?php echo htmlspecialchars($ext_data['title']) ?></span></h3>
					<ul class="field">
						<li><strong class="alabel"><?php echo $lang_admin['Extension version'] ?></strong> <span class="input"><?php echo $ext_data['version'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension author'] ?></strong> <span class="input"><?php echo $ext_data['author'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension description'] ?></strong> <span class="input"><?php echo htmlspecialchars($ext_data['description']) ?></span></li>
					</ul>
				</div>

				<p class="field">
					<label for="fld1"><?php echo $lang_admin['Keep data'] ?></label><br />
					<span class="input"><input type="checkbox" id="fld1" name="keep_data" /><br /> <em class="field-info"><?php echo $lang_admin['Keep data info'] ?></em></span>
				</p>
				<p class="formsubmit"><input type="submit" name="uninstall[<?php echo $id ?>]" value="<?php echo $lang_admin['Uninstall'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

		require PUN_ROOT.'footer.php';
	}

	if (isset($_POST['keep_data']))
		define('KEEP_DATA', true);
	else
		define('KEEP_DATA', false);
	$notes = array();

	// Run uninstall code
	eval($ext_data['uninstall']);

	// Now delete the extension and its hooks from the db
	$db->query('DELETE FROM '.$db->prefix.'extension_hooks WHERE extension_id=\''.$db->escape($id).'\'') or error('Unable to delete extension hooks', __FILE__, __LINE__, $db->error());
	$db->query('DELETE FROM '.$db->prefix.'extensions WHERE id=\''.$db->escape($id).'\'') or error('Unable to delete extension', __FILE__, __LINE__, $db->error());

	// Empty the PHP cache
	$d = dir(PUN_ROOT.'cache');
	while (($entry = $d->read()) !== false)
	{
		if (substr($entry, strlen($entry)-4) == '.php')
			@unlink(PUN_ROOT.'cache/'.$entry);
	}
	$d->close();

	// Regenerate the hooks cache
	require_once PUN_ROOT.'include/cache.php';
	generate_hooks_cache();

	// Display notes if there's any
	if (!empty($notes))
	{
		// Setup breadcrumbs
		$pun_page['crumbs'] = array(
			array($pun_config['o_board_title'], pun_link($pun_url['index'])),
			array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
			$lang_admin['Manage extensions']
		);

		define('PUN_PAGE_SECTION', 'extensions');
		define('PUN_PAGE', 'admin-extensions');

		require PUN_ROOT.'header.php';
?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="form-warn">
				<p><strong><?php echo $lang_admin['Install notes'] ?></strong></p>
				<ul class="warnlist">
<?php

	while (list(, $cur_note) = each($notes))
		echo "\t\t\t\t\t".'<li><span>'.$cur_note.'</span></li>'."\n";

?>
				</ul>
			</div>
		</div>
	</div>

</div>
<?php
			require PUN_ROOT.'footer.php';
	}
	else
		redirect(pun_link($pun_url['admin_extensions']), $lang_admin['Extension uninstalled'].' '.$lang_admin['Redirect']);
}


// Generate an array of installed extensions
$inst_exts = array();
$result = $db->query('SELECT * FROM '.$db->prefix.'extensions ORDER BY title') or error('Unable to fetch installed extensions', __FILE__, __LINE__, $db->error());
while ($cur_ext = $db->fetch_assoc($result))
	$inst_exts[$cur_ext['id']] = $cur_ext;

// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
	$lang_admin['Manage extensions']
);

define('PUN_PAGE_SECTION', 'extensions');
define('PUN_PAGE', 'admin-extensions');

require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="panel">
				<div class="form-intro">
					<h2><span><?php echo $lang_admin['Extensions available'] ?></span></h2>
				</div>
<?php

// Make sure we have XML support
if (function_exists('xml_parser_create'))
{
	echo "\t\t\t\t".'<div class="textset">'."\n";
	$num_exts = 0;
	$num_failed = 0;
	$d = dir(PUN_ROOT.'extensions');
	while (($entry = $d->read()) !== false)
	{

		if ($entry{0} != '.' && is_dir(PUN_ROOT.'extensions/'.$entry))
		{
			if (preg_match('/[^a-zA-Z_]/', $entry))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.$lang_admin['Illegal ID'].'</span></p>'."\n";
				++$num_failed;
				continue;
			}
			else if (!file_exists(PUN_ROOT.'extensions/'.$entry.'/manifest.xml'))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.$lang_admin['Missing manifest'].'</span></p>'."\n";
				++$num_failed;
				continue;
			}

			// Parse manifest.xml into an array
			$ext_data = xml_to_array(@file_get_contents(PUN_ROOT.'extensions/'.$entry.'/manifest.xml'));
			if (empty($ext_data))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.$lang_admin['Failed parse manifest'].'</span></p>'."\n";
				++$num_failed;
				continue;
			}

			// Validate manifest
			$errors = validate_manifest($ext_data, $entry);
			if (!empty($errors))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.implode(' ', $errors).'</span></p>'."\n";
				++$num_failed;
			}
			else
			{
				if (!array_key_exists($entry, $inst_exts) || version_compare($inst_exts[$entry]['version'], $ext_data['extension']['version'], '!='))
				{
					echo "\t\t\t\t\t".'<p class="field"><strong class="alabel"><a href="'.pun_link($pun_url['admin_extensions']).'?install='.urlencode($entry).'">'.$lang_admin['Install'].'</a></strong> <span class="input"><strong>'.htmlspecialchars($ext_data['extension']['title']).' '.$ext_data['extension']['version'].'</strong></span></p>'."\n";
					++$num_exts;
				}
			}
		}
	}
	$d->close();

	echo "\t\t\t\t".'</div>';

	if (!$num_exts)
	{
		if (!$num_failed)
			echo "\t\t\t\t".'<p>'.$lang_admin['No available extensions'].'</p>'."\n";
		else
			echo "\t\t\t\t".'<div class="form-warn"><p><strong>'.$lang_admin['Important'].'</strong> '.$lang_admin['No valid extensions'].'</p></div>';
	}
	else
	{
		if ($num_failed)
			echo "\t\t\t\t".'<div class="form-warn"><p><strong>'.$lang_admin['Important'].'</strong> '.$lang_admin['Some invalid extensions'].'</p></div>';
	}
}
else
	echo "\t\t\t\t".'<p>'.$lang_admin['No XML support'].'</p>'."\n";

?>
			</div>
<?php

if (!empty($inst_exts))
{

?>
			<div class="divider"><hr /></div>
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2><span><?php echo $lang_admin['Installed extensions'] ?></span></h2>
				</div>
				<div class="form-warn"><p><?php echo '<strong>'.$lang_admin['Warning'].'</strong> '.$lang_admin['Installed extensions warn'] ?></p></div>
			</div>
<?php

	while (list($id, $ext) = @each($inst_exts))
	{

?>
			<div class="panel subpanel">
				<div class="textset">
					<h3 class="field"><strong class="alabel"><?php echo $lang_admin['Extension title'] ?><span>:</span></strong> <span class="input"><?php echo htmlspecialchars($ext['title']) ?></span></h3>
					<ul class="field">
						<li><strong class="alabel"><?php echo $lang_admin['Extension version'] ?><span>:</span></strong> <span class="input"><?php echo $ext['version'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension author'] ?><span>:</span></strong> <span class="input"><?php echo $ext['author'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension description'] ?><span>:</span></strong> <span class="input"><?php echo htmlspecialchars($ext['description']) ?></span></li>
					</ul>
				</div>
				<p class="formsubmit"><input type="submit" name="uninstall[<?php echo $id ?>]" value="<?php echo $lang_admin['Uninstall'] ?>" /></p>
			</div>
<?php

	}
	echo "\t\t\t".'</form>'."\n";
}
else
	echo "\t\t\t".'<div class="panel">'."\n\t\t\t\t".'<div class="form-intro">'."\n\t\t\t\t\t".'<h2><span>'.$lang_admin['Installed extensions'].'</span></h2>'."\n\t\t\t\t".'</div>'."\n\t\t\t\t".'<p>'.$lang_admin['No installed extensions'].'</p>'."\n\t\t\t".'</div>'."\n";

?>
		</div>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
