<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', '../');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';


if ($pun_user['g_id'] != PUN_ADMIN)
	message($lang_common['No permission']);

// Load the admin.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/admin.php';


if (isset($_GET['action']) || isset($_POST['prune']) || isset($_POST['prune_comply']))
{
	if (isset($_POST['prune_comply']))
	{
		$prune_from = $_POST['prune_from'];
		$prune_days = intval($_POST['prune_days']);
		$prune_date = ($prune_days) ? time() - ($prune_days*86400) : -1;

		@set_time_limit(0);

		if ($prune_from == 'all')
		{
			$result = $db->query('SELECT id FROM '.$db->prefix.'forums') or error('Unable to fetch forum list', __FILE__, __LINE__, $db->error());
			$num_forums = $db->num_rows($result);

			for ($i = 0; $i < $num_forums; ++$i)
			{
				$fid = $db->result($result, $i);

				prune($fid, $_POST['prune_sticky'], $prune_date);
				update_forum($fid);
			}
		}
		else
		{
			$prune_from = intval($prune_from);
			prune($prune_from, $_POST['prune_sticky'], $prune_date);
			update_forum($prune_from);
		}

		// Locate any "orphaned redirect topics" and delete them
		$result = $db->query('SELECT t1.id FROM '.$db->prefix.'topics AS t1 LEFT JOIN '.$db->prefix.'topics AS t2 ON t1.moved_to=t2.id WHERE t2.id IS NULL AND t1.moved_to IS NOT NULL') or error('Unable to fetch redirect topics', __FILE__, __LINE__, $db->error());
		$num_orphans = $db->num_rows($result);

		if ($num_orphans)
		{
			for ($i = 0; $i < $num_orphans; ++$i)
				$orphans[] = $db->result($result, $i);

			$db->query('DELETE FROM '.$db->prefix.'topics WHERE id IN('.implode(',', $orphans).')') or error('Unable to delete redirect topics', __FILE__, __LINE__, $db->error());
		}

		redirect(pun_link($pun_url['admin_prune']), 'Posts pruned. Redirecting …');
	}


	$prune_days = intval($_POST['req_prune_days']);
	if ($prune_days < 0)
		message($lang_admin['Days to prune message']);

	$prune_date = time() - ($prune_days*86400);
	$prune_from = $_POST['prune_from'];

	// Concatenate together the query for counting number or topics to prune
	$sql = 'SELECT COUNT(id) FROM '.$db->prefix.'topics WHERE last_post<'.$prune_date.' AND moved_to IS NULL';

	if ($_POST['prune_sticky'] == '0')
		$sql .= ' AND sticky=\'0\'';

	if ($prune_from != 'all')
	{
		$prune_from = intval($prune_from);
		$sql .= ' AND forum_id='.$prune_from;

		// Fetch the forum name (just for cosmetic reasons)
		$result = $db->query('SELECT forum_name FROM '.$db->prefix.'forums WHERE id='.$prune_from) or error('Unable to fetch forum name', __FILE__, __LINE__, $db->error());
		$forum = htmlspecialchars($db->result($result));
	}
	else
		$forum = 'all forums';

	$result = $db->query($sql) or error('Unable to fetch topic prune count', __FILE__, __LINE__, $db->error());
	$num_topics = $db->result($result);

	if (!$num_topics)
		message($lang_admin['No days old message']);


	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
		array($lang_admin['Prune old posts'], pun_link($pun_url['admin_prune'])),
		$lang_admin['Confirm prune heading']
	);

	define('PUN_PAGE_SECTION', 'management');
	define('PUN_PAGE', 'admin-prune');
	require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_prune']) ?>?action=foo">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="prune_days" value="<?php echo $prune_days ?>" />
				<input type="hidden" name="prune_sticky" value="<?php echo $_POST['prune_sticky'] ?>" />
				<input type="hidden" name="prune_from" value="<?php echo $prune_from ?>" />
				<div class="form-intro">
					<h2><span><?php echo end($pun_page['crumbs']) ?></span></h2>
				</div>
				<ul class="textset" id="fs1">
					<li class="field"><strong class="alabel"><?php echo $lang_admin['Days old'] ?><span>:</span></strong> <span class="input"><?php echo $prune_days ?></span></li>
					<li class="field"><strong class="alabel"><?php echo $lang_admin['Forum'] ?><span>:</span></strong> <span class="input"><?php echo ($forum == 'all forums') ? $lang_admin['All forums'] : $forum ?></span></li>
					<li class="field"><strong class="alabel"><?php echo $lang_admin['Prune sticky'] ?><span>:</span></strong> <span class="input"><?php echo ($_POST['prune_sticky'] == '1') ? $lang_admin['Yes'] : $lang_admin['No'] ?></span></li>
					<li class="field"><strong class="alabel"><?php echo $lang_admin['Number of topics'] ?><span>:</span></strong> <span class="input"><?php echo $num_topics ?></span></li>
				</ul>
				<div class="form-warn">
					<p><strong class="warntext"><?php echo $lang_admin['Prune warning'] ?></strong></p>
				</div>

				<p class="formsubmit"><input type="submit" class="button" name="prune_comply" value="<?php echo $lang_admin['Prune old posts'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

	require PUN_ROOT.'footer.php';
}


else
{
	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
		$lang_admin['Prune old posts']
	);

	define('PUN_PAGE_SECTION', 'management');
	define('PUN_PAGE', 'admin-prune');
	require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_prune']) ?>?action=foo">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="form_sent" value="1" />
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Prune settings'] ?>: </span></h2>
					<p><?php echo $lang_admin['Prune intro'] ?></p>
				</div>
				<p id="req-msg" class="req-warn"><?php printf($lang_common['Required warn'], '<em class="req-text">'.$lang_common['Required mark'].'<span> '.$lang_common['Required'].'</span></em>') ?></p>
				<fieldset class="textset" id="fs1">
					<legend><span><?php echo $lang_admin['Prune legend'] ?></span></legend>
					<p class="field">
						<label for="fs1f1"><?php echo $lang_admin['Prune from'] ?></label><br />
						<span class="input"><select id="fs1f1" name="prune_from">
							<option value="all"><?php echo $lang_admin['All forums'] ?></option>
<?php

	$result = $db->query('SELECT c.id AS cid, c.cat_name, f.id AS fid, f.forum_name FROM '.$db->prefix.'categories AS c INNER JOIN '.$db->prefix.'forums AS f ON c.id=f.cat_id WHERE f.redirect_url IS NULL ORDER BY c.disp_position, c.id, f.disp_position') or error('Unable to fetch category/forum list', __FILE__, __LINE__, $db->error());

	$cur_category = 0;
	while ($forum = $db->fetch_assoc($result))
	{
		if ($forum['cid'] != $cur_category)	// Are we still in the same category?
		{
			if ($cur_category)
				echo "\t\t\t\t\t\t\t\t".'</optgroup>'."\n";

			echo "\t\t\t\t\t\t\t\t".'<optgroup label="'.htmlspecialchars($forum['cat_name']).'">'."\n";
			$cur_category = $forum['cid'];
		}

		echo "\t\t\t\t\t\t\t\t\t".'<option value="'.$forum['fid'].'">'.htmlspecialchars($forum['forum_name']).'</option>'."\n";
	}

?>
						</optgroup>
						</select></span>
					</p>
					<p class="field required">
						<label for="fs1f2"><?php echo $lang_admin['Days old'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="text" id="fs1f2" name="req_prune_days" size="3" maxlength="3" /></span>
					</p>
					<p class="field">
						<span class="input"><input type="checkbox" id="fs1f3" name="prune_sticky" value="1" checked="checked" /> <label for="fs1f3"><?php echo $lang_admin['Prune sticky enable'] ?></label></span><br />
					</p>
				</fieldset>
				<div class="form-warn">
					<p><?php echo '<strong>'.$lang_admin['Important'].'</strong>'.' '.$lang_admin['Prune caution'] ?></p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="prune" value="<?php echo $lang_admin['Prune old posts'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

	require PUN_ROOT.'footer.php';
}
